%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Performs MS-GLM fit to Binomial data using RDP framework.
%% Inputs: dmat  := data matrix (nx2), n=2^J.
%%                  First column are observed number of successes,
%%                  second column is the number of trials from which
%%                  those successes were obtained.
%%            m  := degree polynomials to use 
%%                 (i.e., 0 = constant; 1 = linear; etc.)
%% Output: est  := estimate
%%         splitvec := vector of split points.
%% Calls:  Only internal Matlab functions called.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [est, splitvec]  = MSGLMBinom_RDP(dmat,m)

n = length(dmat(:,1));
J = log2(n);
tt = [0:1:n-1]./n;

xJ = dmat(:,1);
nJ = dmat(:,2);

%% This is the threshold needed for the RP method.
%lam = 2*((1.5+m/2)*log(n));
%% For the RDP method, the following reduced threshold can be used.
lam = 2*((0.5+m/2)*log(n));

% Iterate through scales, from fine to coarse, building a decorated
% tree.  Decorations are 1/0, indicating "split/no split", depending on
% whether location-scale penalized LR test favored a split or not.
decor = [];			% Vector of 0/1's.
bestPL = [];			% Best penalized likelihood values.
bestFit = zeros(size(xJ));	% Best fitted values.

% Initialize fits, with width of starting interval size dependent on
% degree of polynomials being fit.
Js = J - ceil(log2(m+1));
for k=0:1:2^(Js)-1,
  xind = [2^(J-Js)*k + 1: 2^(J-Js)*(k+1)];
  x = xJ(xind);  nn = nJ(xind); tx = tt(xind); 
  if ( length(x) == (m+1)),
    fit = x./nn;
  else,
    txmat = fliplr(vander(tx));  txmat = txmat(:,[1:m+1]);
    beta = glmfit(txmat,[x,nn],'binomial','logit',[],[],[],'off');
    fit  = glmval(beta,txmat,'logit',[],[],nn,[],'off');  fit = fit./nn;
  end
  pl   = gammaln(nn+1) - gammaln(x+1) - gammaln(nn-x+1) + ...
         x.*log(fit+realmin) + (nn-x).*log(1-fit+realmin);
  pl   = sum(pl);
  bestFit(xind) = fit;
  bestPL = [bestPL pl];
end

% Now proceed down through coarser scales.
for j=Js-1:-1:0,
  decorj = [];
  oldbestPL = bestPL;  bestPL = [];
  for k=0:1:2^j-1,
    xind = [2^(J-j)*k + 1: 2^(J-j)*(k+1)];
    plind = [2*k+1, 2*(k+1)];
    x = xJ(xind);  nn = nJ(xind); tx = tt(xind); 
    txmat = fliplr(vander(tx));  txmat = txmat(:,[1:m+1]);
    beta = glmfit(txmat,[x,nn],'binomial','logit',[],[],[],'off');
    fit0 = glmval(beta,txmat,'logit',[],[],nn,[],'off');  fit0 = fit0./nn;
    pl0 = gammaln(nn+1) - gammaln(x+1) - gammaln(nn-x+1) + ...
          x.*log(fit0+realmin) + (nn-x).*log(1-fit0+realmin);
    pl0 = sum(pl0);
    pl1 = sum( oldbestPL(plind) ) - lam*(m+1);
    tempdecor = (pl1 > pl0);
    bestFit(xind) = bestFit(xind).*tempdecor + (fit0).*(1-tempdecor);
    bestPL = [bestPL (pl1.*tempdecor + pl0.*(1-tempdecor)) ];
    decorj = [decorj tempdecor];
  end
  decor = [decorj decor];
end

est = bestFit;

% Postprocess to propogate hereditary structure back up tree.
splitvec=[];
splitvec(1) = decor(1); newdecor(1) = decor(1);
for j=1:1:Js-1,
  temp = reshape([newdecor; newdecor],1,2^j);
  newdecor = temp.*decor(2^j:2^(j+1)-1);
  splitvec = [splitvec newdecor];
end
  
%% Copyright (c) Eric Kolaczyk & Rob Nowak, 2002.
