%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Performs MS-GLM fit to Poisson data using RDP framework.
%% Inputs: sig  := signal vector (1xn), n=2^J.
%%           m  := degree polynomials to use 
%%                 (i.e., 0 = constant; 1 = linear; etc.)
%% Output: est  := estimate
%%         splitvec := vector of split points.
%% Calls:  Only internal Matlab functions called.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [est, splitvec]  = MSGLMPois_RDP(sig,m)

n = length(sig);
J = log2(n);
tt = [0:1:n-1]./n;

xJ = sig;

%% This is the threshold used for the RP method.
%lam = 2*((1.5+m/2)*log(n));
%% For the RDP method, the following reduced threshold may be used.
lam = 2*((0.5+m/2)*log(n));

% Iterate through scales, from fine to coarse, building a decorated
% tree.  Decorations are 1/0, indicating "split/no split", depending on
% whether location-scale penalized LR test favored a split or not.
decor = [];			% Vector of 0/1's.
bestPL = [];			% Best penalized likelihood values.
bestFit = zeros(size(xJ));	% Best fitted values.

% Initialize fits, with width of starting interval size dependent on
% degree of polynomials being fit.
Js = J - ceil(log2(m+1));
for k=0:1:2^(Js)-1,
  xind = [2^(J-Js)*k + 1: 2^(J-Js)*(k+1)];
  x = xJ(xind);  tx = tt(xind);
  txmat = fliplr(vander(tx));  txmat = txmat(:,[1:m+1]);
  beta = glmfit(txmat,x,'poisson','log',[],[],[],'off');
  fit  = glmval(beta,txmat,'log',[],[],[],[],'off');
  pl   = x*(log(fit)) - sum(fit);	%pl = pl - (J-Js)*lam;
  bestFit(xind) = fit;
  bestPL = [bestPL pl];
end

% Now proceed down through coarser scales.
for j=Js-1:-1:0,
  decorj = [];
  oldbestPL = bestPL;  bestPL = [];
  for k=0:1:2^j-1,
    xind = [2^(J-j)*k + 1: 2^(J-j)*(k+1)];
    plind = [2*k+1, 2*(k+1)];
    x = xJ(xind);  tx = tt(xind); 
    txmat = fliplr(vander(tx));  txmat = txmat(:,[1:m+1]);
    beta = glmfit(txmat,x,'poisson','log',[],[],[],'off');
    fit0 = glmval(beta,txmat,'log',[],[],[],[],'off');
    pl0 = x*log(fit0) - sum(fit0);
    pl1 = sum( oldbestPL(plind) ) - lam;
    tempdecor = (pl1 > pl0);
    bestFit(xind) = bestFit(xind).*tempdecor + (fit0)'.*(1-tempdecor);
    bestPL = [bestPL (pl1.*tempdecor + pl0.*(1-tempdecor)) ];
    decorj = [decorj tempdecor];
  end
  decor = [decorj decor];
end

est = bestFit;

% Postprocess to propogate hereditary structure back up tree.
splitvec=[];
splitvec(1) = decor(1); newdecor(1) = decor(1);
for j=1:1:Js-1,
  temp = reshape([newdecor; newdecor],1,2^j);
  newdecor = temp.*decor(2^j:2^(j+1)-1);
  splitvec = [splitvec newdecor];
end
  
%% Copyright (c) Eric Kolaczyk & Rob Nowak, 2002.
