%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Name:  MSGLMPois_RP.m	Created:  04/26/02
%
% Usage: Compute multiscale generalized linear model fit to a Poisson
%        time series, based on the material in the paper
%		Kolaczyk, E.D. and Nowak, R.D. (2002).  
%
% Inputs: x := vector of independent Poisson counts (n x 1),
%	       where n need not be a power of 2.
%         m := degree polynomials to use 
%              (i.e., 0 = constant; 1 = linear; etc.)
% Outputs: bp := vector with locations of optimal segmentation (break) points,
%		 with elements between 1 and n.
%          est := estimate of intensity underlying the Poisson count
%		  data, created by fitting a piecewise polynomial to the
%		  data with with splits at the optimal break points.
% Calls:  TriangleAlgPois.m, GetBreakpoints.m
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [bp, est] = MSGLMPois_RP(x,m)

n = length(x);

%% Algorithm consists of three steps:
%   (i) Computing optimal local (in scale and position) penalized likelihoods
%       for intervals of all sizes 1 to n.  This may be done in a
%       recursive, pyramidal fashion and stored in a set of 
%       triangular arrays.
%  (ii) Extracting from the information in the arrays the segmentation
%       that is actually optimal.
% (iii) Calculating the estimate for the underlying intensity on the
%       optimal segmentation. 

% Run triangle algorithm.
[t0, t1, mm] = TriangleAlgPois(x,m);

% Extract breakpoints.
bp = [];
bp = GetBreakpoints(bp,1,n,t0,t1,mm);
bp = sort(bp);

% Compute piecewise polynomial estimator from data and breakpoint info.
tt = [0:1:n-1]./n;
bp = [0 bp];
if bp == 0,
  xx = x;  tx = tt;
  txmat = fliplr(vander(tx));  txmat = txmat(:,[1:m+1]);
  beta = glmfit(txmat,xx,'poisson','log',[],[],[],'off');
  est  = glmval(beta,txmat,'log',[],[],[],[],'off');
else
  for i=2:length(bp)
    if (bp(i)-bp(i-1))<= (m+1),
      est(bp(i-1)+1:bp(i)) = x(bp(i-1)+1:bp(i));
    else,
      xx = x(bp(i-1)+1:bp(i)); tx = tt(bp(i-1)+1:bp(i));
      txmat = fliplr(vander(tx));  
      txmat = txmat(:,[1:m+1]);
      beta = glmfit(txmat,xx,'poisson','log',[],[],[],'off');
      est(bp(i-1)+1:bp(i))  = glmval(beta,txmat,'log',[],[],[],[],'off');
    end
  end
  if (length(x)-bp(i))<= (m+1),
    est(bp(i)+1:length(x)) = x(bp(i)+1:length(x));
  else,
    xx = x(bp(i)+1:length(x)); tx = tt(bp(i)+1:length(x));
    txmat = fliplr(vander(tx));  txmat = txmat(:,[1:m+1]);
    beta = glmfit(txmat,xx,'poisson','log',[],[],[],'off');
    est(bp(i)+1:length(x))  = glmval(beta,txmat,'log',[],[],[],[],'off');
  end
end

% Plot the final results, both estimator and breakpoints.
%clf
%plot(tt,x,'b*')
%hold on
%plot(tt,est,'r')
%if length(bp) > 0,
%  Msig = 1.10*max(x); msig = min(x);
%  for i=2:length(bp),
%    bploc = tt(bp(i)) + (1/(2*n));
%    line([bploc, bploc], [msig,Msig])
%  end
%end

%  This code has been developed by Eric Kolaczyk and Robert Nowak.
%  Copyright (c) Eric Kolaczyk and Robert Nowak, 2002.
%  All rights reserved.
