%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Name:  TriangleAlgBinom.m    Created:  05/20/02
%
% Usage: Performs a recursive, pyramidal series of local (in scale 
%        and position) penalized likelihood optimizations.  Essentially,
%        (log) probabilities for the two hypotheses of "one-piece poly" or
%        "two-piece poly" are computed for all substrings of the data
%        vector, starting with length 1 and building recursively to
%        the full length n.  These are stored in two separate triangular
%        arrays.  Each row of the arrays corresponds to substrings (or 'blocks')
%        of data of length/scale 1,2,...,n.  Additionally, a mask is built 
%        along the way to indicate the positions of 'split' decisions at each
%        scale.
%
% Inputs: dmat := data matrix (nx2), where n need not be a power of 2.
%	              First column are observed x; second column are n's.
%         m := degree polynomials to use 
%              (i.e., 0 = constant; 1 = linear; etc.)
% Outputs: t0 := triangular array of optimal objective function values 
%                under null hypothesis (i.e., "one-piece poly").
%          t1 := triangular array of optimal objective function values
%                under alt hypothesis (i.e., "two-piece poly").
%          mm := mask indicating (with a '1') the decided split 
%		 locations in the sequence (in relation to the  
%                arrays t0 and t1).
%
% Calls:  Only internal Matlab functions called.  Requires the
%         Statistics toolbox to be added to the basic Matlab package.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [t0,t1,mm] = TriangleAlgBinom(dmat,m);

% Some preliminary stuff.

n = length(dmat(:,1));
tt = [0:1:n-1]./n;
lam = 2*((1.5+m/2)*log(n));

x = dmat(:,1);
nn = dmat(:,2);

for j=1:m+1,
  for i=1:n-j+1,
    xx = x(i:i+j-1);  nnn = nn(i:i+j-1);
    fit = xx./nnn;
    pl = gammaln(nnn+1) - gammaln(xx+1) - gammaln(nnn-xx+1) + ...
         xx.*log(fit+realmin) + (nnn-xx).*log(1-fit+realmin);
    pl   = sum(pl);
    t0(i,i+j-1) = pl;  
    t1(i,i+j-1) = pl;
    mm(i,i+j-1) = 0;
  end
end

% Begin iterative calculation of penalized likelihoods over intervals of 
% increasing size, storing the various results in the upper triangles
% of the matrices t0, t1, and mm.
for j=m+2:n,
  for i=1:n-j+1,
    xx = x(i:i+j-1); nnn = nn(i:i+j-1); tx = tt(i:i+j-1);
    txmat = fliplr(vander(tx));  txmat = txmat(:,[1:m+1]);
    beta = glmfit(txmat,[xx,nnn],'binomial','logit',[],[],[],'off');
    fit  = glmval(beta,txmat,'logit',[],[],nnn,[],'off');  fit = fit./nnn;
    pl0  = gammaln(nnn+1) - gammaln(xx+1) - gammaln(nnn-xx+1) + ...
           xx.*log(fit+realmin) + (nnn-xx).*log(1-fit+realmin);
    pl0  = sum(pl0);
    pl1  = t1(i,i+(1:j-1)-1) + t1(i+(1:j-1),i+j-1)' - lam;
    t0(i,i+j-1) = pl0;
    t1(i,i+j-1) = max(max(pl1),pl0);
    [dummy,mm(i,i+j-1)] = max(pl1);
    mm(i,i+j-1) = mm(i,i+j-1)+i-1;
  end
end

%  This code has been developed by Eric Kolaczyk and Robert Nowak.
%  Copyright (c) Eric Kolaczyk and Robert Nowak, 2002.
%  All rights reserved.
