%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Name:  TriangleAlgPois.m    Created:  04/26/02
%
% Usage: Performs a recursive, pyramidal series of local (in scale 
%        and position) penalized likelihood optimizations.  Essentially,
%        (log) probabilities for the two hypotheses of "one-piece poly" or
%        "two-piece poly" are computed for all substrings of the data
%        vector, starting with length 1 and building recursively to
%        the full length n.  These are stored in two separate triangular
%        arrays.  Each row of the arrays corresponds to substrings (or 'blocks')
%        of data of length/scale 1,2,...,n.  Additionally, a mask is built 
%        along the way to indicate the positions of 'split' decisions at each
%        scale.
%
% Inputs: x := vector of independent Poisson counts (n x 1),
%              where n need not be a power of 2.
%         m := degree polynomials to use 
%              (i.e., 0 = constant; 1 = linear; etc.)
% Outputs: t0 := triangular array of optimal objective function values 
%                under null hypothesis (i.e., "one-piece poly").
%          t1 := triangular array of optimal objective function values
%                under alt hypothesis (i.e., "two-piece poly").
%          mm := mask indicating (with a '1') the decided split 
%		 locations in the sequence (in relation to the  
%                arrays t0 and t1).
%
% Calls:  Only internal Matlab functions called.  Requires the
%         Statistics toolbox to be added to the basic Matlab package.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [t0,t1,mm] = TriangleAlgPois(x,m);

% Some preliminary stuff.

n = length(x);
tt = [0:1:n-1]./n;
lam = 2*((1.5+m/2)*log(n));

for j=1:m+1,
  for i=1:n-j+1,
    xx = x(i:i+j-1); 
    pl   = (xx)*(log(xx))' - sum(xx);
    t0(i,i+j-1) = pl;  
    t1(i,i+j-1) = pl;
    mm(i,i+j-1) = 0;
  end
end

% Begin iterative calculation of penalized likelihoods over intervals of 
% increasing size, storing the various results in the upper triangles
% of the matrices t0, t1, and mm.
for j=m+2:n,
  for i=1:n-j+1,
    xx = x(i:i+j-1); tx = tt(i:i+j-1);
    txmat = fliplr(vander(tx));  txmat = txmat(:,[1:m+1]);
    beta = glmfit(txmat,xx,'poisson','log',[],[],[],'off');
    fit  = glmval(beta,txmat,'log',[],[],[],[],'off');
    pl0  = xx*(log(fit)) - sum(fit); 
    pl1  = t1(i,i+(1:j-1)-1) + t1(i+(1:j-1),i+j-1)' - lam;
    t0(i,i+j-1) = pl0;
    t1(i,i+j-1) = max(max(pl1),pl0);
    [dummy,mm(i,i+j-1)] = max(pl1);
    mm(i,i+j-1) = mm(i,i+j-1)+i-1;
  end
end

%  This code has been developed by Eric Kolaczyk and Robert Nowak.
%  Copyright (c) Eric Kolaczyk and Robert Nowak, 2002.
%  All rights reserved.
